@@ -1,15 +1,13 @@
-import _patch   # NOQA
 import logging
 import os
 import warnings
 
 
-version_info = (0, 9, 2)
+version_info = (0, 10, 0)
 __version__ = ".".join(map(str, version_info))
 
 
 logger = logging.getLogger('circus')
-logging.basicConfig()
 
 
 class ArbiterHandler(object):
@@ -20,8 +18,9 @@ class ArbiterHandler(object):
                  statsd_close_outputs=False,
                  multicast_endpoint=None,
                  env=None, name=None, context=None,
-                 background=False, stream_backend="thread",
-                 plugins=None, debug=False, proc_name="circusd"):
+                 background=False, stream_backend="thread", httpd=False,
+                 plugins=None, debug=False, proc_name="circusd",
+                 loop=None, check_delay=1.0):
         """Creates a Arbiter and a single watcher in it.
 
         Options:
@@ -57,8 +56,6 @@ class ArbiterHandler(object):
 
                 - **class**: the fully qualified name of the class to use for
                              streaming. Defaults to circus.stream.FileStream
-                - **refresh_time**: the delay between two stream checks.
-                                    Defaults to 0.3 seconds.
                 - any other key will be passed the class constructor.
             - **stderr_stream**: a mapping containing the options for
               configuring the stderr stream. Default to None. When provided,
@@ -66,8 +63,6 @@ class ArbiterHandler(object):
 
                 - **class**: the fully qualified name of the class to use for
                   streaming. Defaults to circus.stream.FileStream
-                - **refresh_time**: the delay between two stream checks.
-                                    Defaults to 0.3 seconds.
                 - any other key will be passed the class constructor.
             - **max_retry**: the number of times we attempt to start a process,
               before we abandon and stop the whole watcher. (default: 5)
@@ -77,7 +72,7 @@ class ArbiterHandler(object):
               callable or the callabled itself and a boolean flag indicating if
               an exception occuring in the hook should not be ignored.
               Possible values for the hook name: *before_start*, *after_start*,
-              *before_stop*, *after_stop*.
+              *before_stop*, *after_stop*, *before_signal*, *after_signal*
 
         - **controller** -- the zmq entry point
           (default: 'tcp://127.0.0.1:5555')
@@ -102,6 +97,9 @@ class ArbiterHandler(object):
         - **debug** -- If True the arbiter is launched in debug mode
           (default: False)
         - **proc_name** -- the arbiter process name (default: circusd)
+        - **loop** -- the event loop (default: None)
+        - **check_delay** -- the delay between two controller points
+        (default: 1 s)
         """
         from circus.util import (DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_SUB,
                                  DEFAULT_ENDPOINT_MULTICAST,
@@ -133,12 +131,12 @@ class ArbiterHandler(object):
             _watchers.append(Watcher.load_from_config(watcher))
 
         return Arbiter(_watchers, controller, pubsub_endpoint,
-                       statsd=statsd,
+                       httpd=httpd, statsd=statsd,
                        stats_endpoint=stats_endpoint,
                        statsd_close_outputs=statsd_close_outputs,
                        multicast_endpoint=multicast_endpoint,
                        context=context, plugins=plugins, debug=debug,
-                       proc_name=proc_name)
+                       proc_name=proc_name, loop=loop, check_delay=check_delay)
 
     def _get_arbiter_klass(self, background):
         if background:
