@@ -1,18 +1,20 @@
 import sys
 import argparse
+import logging
 import os
 import resource
 
 from circus import logger
-from circus.arbiter import Arbiter, ReloadArbiterException
+from circus.arbiter import Arbiter
 from circus.pidfile import Pidfile
 from circus import __version__
 from circus.util import MAXFD, REDIRECT_TO, configure_logger, LOG_LEVELS
+from circus.util import check_future_exception_and_log
 
 
 def get_maxfd():
     maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
-    if (maxfd == resource.RLIM_INFINITY):
+    if maxfd == resource.RLIM_INFINITY:
         maxfd = MAXFD
     return maxfd
 
@@ -22,7 +24,7 @@ try:
 except ImportError:
     def closerange(fd_low, fd_high):    # NOQA
         # Iterate through and close all file descriptors.
-        for fd in xrange(fd_low, fd_high):
+        for fd in range(fd_low, fd_high):
             try:
                 os.close(fd)
             except OSError:    # ERROR, fd wasn't open to begin with (ignored)
@@ -38,14 +40,20 @@ def daemonize():
         if module.startswith('gevent'):
             raise ValueError('Cannot daemonize if gevent is loaded')
 
-    #if not 'CIRCUS_PID' in os.environ:
-    if os.fork():
+    child_pid = os.fork()
+
+    if child_pid != 0:
+        # we're in the parent
         os._exit(0)
+
+    # child process
     os.setsid()
 
-    if os.fork():
+    subchild = os.fork()
+    if subchild:
         os._exit(0)
 
+    # subchild
     os.umask(0)
     maxfd = get_maxfd()
     closerange(0, maxfd)
@@ -61,8 +69,8 @@ def main():
 
     # XXX we should be able to add all these options in the config file as well
     parser.add_argument('--log-level', dest='loglevel',
-                        choices=LOG_LEVELS.keys() + [key.upper() for key in
-                                                     LOG_LEVELS.keys()],
+                        choices=list(LOG_LEVELS.keys()) + [
+                            key.upper() for key in LOG_LEVELS.keys()],
                         help="log level")
     parser.add_argument('--log-output', dest='logoutput', help=(
         "The location where the logs will be written. The default behavior "
@@ -88,6 +96,14 @@ def main():
     if args.daemonize:
         daemonize()
 
+    # This config call is done to avoid any
+    #     "no handlers could be found for logger"
+    #
+    # error while loding the configuration and setting up the arbiter.
+    # The real logging configuration is done right after via
+    # a configure_logger() call
+    logging.basicConfig()
+
     # From here it can also come from the arbiter configuration
     # load the arbiter from config
     arbiter = Arbiter.load_from_config(args.config)
@@ -98,7 +114,7 @@ def main():
 
         try:
             pidfile.create(os.getpid())
-        except RuntimeError, e:
+        except RuntimeError as e:
             print(str(e))
             sys.exit(1)
 
@@ -107,24 +123,24 @@ def main():
     logoutput = args.logoutput or arbiter.logoutput or '-'
     configure_logger(logger, loglevel, logoutput)
 
-    try:
-        restart_after_stop = True
-        while restart_after_stop:
-            while True:
-                try:
-                    arbiter = Arbiter.load_from_config(args.config)
-                    restart_after_stop = arbiter.start()
-                except ReloadArbiterException:
-                    pass
-                else:
-                    break
-    except KeyboardInterrupt:
-        pass
-    finally:
-        arbiter.stop()
-        if pidfile is not None:
-            pidfile.unlink()
-
+    # Main loop
+    restart = True
+    while restart:
+        try:
+            arbiter = Arbiter.load_from_config(args.config)
+            future = arbiter.start()
+            restart = False
+            if check_future_exception_and_log(future) is None:
+                restart = arbiter._restarting
+        except Exception as e:
+            # emergency stop
+            arbiter.loop.run_sync(arbiter._emergency_stop)
+            raise(e)
+        except KeyboardInterrupt:
+            pass
+        finally:
+            if pidfile is not None:
+                pidfile.unlink()
     sys.exit(0)
 
 
