@@ -17,8 +17,9 @@ class Restart(Command):
 
             {
                 "command": "restart",
-                "propeties": {
-                    "name": '<name>"
+                "properties": {
+                    "name": "<name>",
+                    "waiting": False
                 }
             }
 
@@ -27,35 +28,43 @@ class Restart(Command):
         If the property name is present, then the reload will be applied
         to the watcher.
 
+        If ``waiting`` is False (default), the call will return immediatly
+        after calling `stop_signal` on each process.
+
+        If ``waiting`` is True, the call will return only when the restart
+        process is completly ended. Because of the
+        :ref:`graceful_timeout option <graceful_timeout>`, it can take some
+        time.
+
 
         Command line
         ------------
 
         ::
 
-            $ circusctl restart [<name>] [--terminate]
+            $ circusctl restart [<name>] [--waiting]
 
         Options
         +++++++
 
         - <name>: name of the watcher
-        - --terminate; quit the node immediately
-
     """
+
     name = "restart"
+    options = Command.waiting_options
 
     def message(self, *args, **opts):
         if len(args) > 1:
             raise ArgumentError("Invalid number of arguments")
 
         if len(args) == 1:
-            return self.make_message(name=args[0])
-        else:
-            return self.make_message()
+            return self.make_message(name=args[0], **opts)
+
+        return self.make_message(**opts)
 
     def execute(self, arbiter, props):
         if 'name' in props:
             watcher = self._get_watcher(arbiter, props['name'])
-            watcher.restart()
+            return watcher.restart()
         else:
-            arbiter.restart()
+            return arbiter.restart(inside_circusd=True)
