@@ -1,13 +1,13 @@
 import glob
 import os
+import signal
 import warnings
 from fnmatch import fnmatch
-from collections import defaultdict
 
 from circus import logger
 from circus.util import (DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_SUB,
                          DEFAULT_ENDPOINT_MULTICAST, DEFAULT_ENDPOINT_STATS,
-                         StrictConfigParser, parse_env_str, replace_gnu_args)
+                         StrictConfigParser, replace_gnu_args, to_signum)
 
 
 def watcher_defaults():
@@ -23,6 +23,8 @@ def watcher_defaults():
         'uid': None,
         'gid': None,
         'send_hup': False,
+        'stop_signal': signal.SIGTERM,
+        'stop_children': False,
         'max_retry': 5,
         'graceful_timeout': 30,
         'rlimits': dict(),
@@ -58,11 +60,6 @@ class DefaultConfigParser(StrictConfigParser):
     def set_env(self, env):
         self._env = dict(env)
 
-    def toboolean(self, value):
-        if value.lower() not in self._boolean_states:
-            raise ValueError('Not a boolean: %s' % value)
-        return self._boolean_states[value.lower()]
-
     def get(self, section, option):
         res = StrictConfigParser.get(self, section, option)
         return replace_gnu_args(res, env=self._env)
@@ -71,8 +68,8 @@ class DefaultConfigParser(StrictConfigParser):
         items = StrictConfigParser.items(self, section)
         if noreplace:
             return items
-        env = dict(os.environ)
-        return [(key, replace_gnu_args(value, env=env))
+
+        return [(key, replace_gnu_args(value, env=self._env))
                 for key, value in items]
 
     def dget(self, section, option, default=None, type=str):
@@ -94,7 +91,10 @@ class DefaultConfigParser(StrictConfigParser):
 def read_config(config_path):
     cfg = DefaultConfigParser()
     with open(config_path) as f:
-        cfg.readfp(f)
+        if hasattr(cfg, 'read_file'):
+            cfg.read_file(f)
+        else:
+            cfg.readfp(f)
 
     current_dir = os.path.dirname(config_path)
 
@@ -136,10 +136,14 @@ def get_config(config_file):
         return [(key.upper(), value) for key, value in items]
 
     global_env = dict(_upper(os.environ.items()))
+    local_env = dict()
 
+    # update environments with [env] section
     if 'env' in cfg.sections():
-        global_env.update(dict(_upper(cfg.items('env'))))
+        local_env.update(dict(_upper(cfg.items('env'))))
+        global_env.update(local_env)
 
+    # always set the cfg environment
     cfg.set_env(global_env)
 
     # main circus options
@@ -179,6 +183,7 @@ def get_config(config_file):
         if section.startswith("socket:"):
             sock = dict(cfg.items(section))
             sock['name'] = section.split("socket:")[-1].lower()
+            sock['so_reuseport'] = dget(section, "so_reuseport", False, bool)
             sockets.append(sock)
 
         if section.startswith("plugin:"):
@@ -216,6 +221,11 @@ def get_config(config_file):
                 elif opt == 'send_hup':
                     watcher['send_hup'] = dget(section, 'send_hup', False,
                                                bool)
+                elif opt == 'stop_signal':
+                    watcher['stop_signal'] = to_signum(val)
+                elif opt == 'stop_children':
+                    watcher['stop_children'] = dget(section, 'stop_children',
+                                                    False, bool)
                 elif opt == 'check_flapping':
                     watcher['check_flapping'] = dget(section, 'check_flapping',
                                                      True, bool)
@@ -258,11 +268,6 @@ def get_config(config_file):
                 elif opt == 'respawn':
                     watcher['respawn'] = dget(section, "respawn", True, bool)
 
-                elif opt == 'env':
-                    logger.warning('the env option is deprecated the use of '
-                                   'env sections is recommended')
-                    watcher['env'] = parse_env_str(val)
-
                 elif opt == 'autostart':
                     watcher['autostart'] = dget(section, "autostart", True,
                                                 bool)
@@ -278,47 +283,55 @@ def get_config(config_file):
                     # freeform
                     watcher[opt] = val
 
+            if watcher['copy_env']:
+                watcher['env'] = dict(global_env)
+            else:
+                watcher['env'] = dict(local_env)
+
             watchers.append(watcher)
 
     # Second pass to make sure env sections apply to all watchers.
-    environs = defaultdict(dict)
 
-    # global env first
     def _extend(target, source):
         for name, value in source:
             if name in target:
                 continue
             target[name] = value
 
-    for watcher in watchers:
-        _extend(environs[watcher['name']], global_env.items())
+    def _expand_vars(target, key, env):
+        if isinstance(target[key], str):
+            target[key] = replace_gnu_args(target[key], env=env)
+        elif isinstance(target[key], dict):
+            for k in target[key].keys():
+                _expand_vars(target[key], k, env)
 
-    # then per-watcher env
+    def _expand_section(section, env, exclude=None):
+        if exclude is None:
+            exclude = ('name', 'env')
+
+        for option in section.keys():
+            if option in exclude:
+                continue
+            _expand_vars(section, option, env)
+
+    # build environment for watcher sections
     for section in cfg.sections():
         if section.startswith('env:'):
             section_elements = section.split("env:", 1)[1]
             watcher_patterns = [s.strip() for s in section_elements.split(',')]
+            env_items = dict(_upper(cfg.items(section, noreplace=True)))
 
             for pattern in watcher_patterns:
                 match = [w for w in watchers if fnmatch(w['name'], pattern)]
 
                 for watcher in match:
-                    watcher_name = watcher['name']
-                    extra = cfg.items(section, noreplace=True)
-                    environs[watcher_name].update(_upper(extra))
+                    watcher['env'].update(env_items)
 
+    # expand environment for watcher sections
     for watcher in watchers:
-        if watcher['name'] in environs:
-            if not 'env' in watcher:
-                watcher['env'] = dict()
-            _extend(watcher['env'], environs[watcher['name']].items())
-
-        for option, value in watcher.items():
-            if option in ('name', 'env'):
-                continue
-            if not isinstance(value, str):
-                continue
-            watcher[option] = replace_gnu_args(value, env=watcher['env'])
+        env = dict(global_env)
+        env.update(watcher['env'])
+        _expand_section(watcher, env)
 
     config['watchers'] = watchers
     config['plugins'] = plugins
