@@ -2,11 +2,15 @@ import sys
 import random
 
 from datetime import datetime
-from Queue import Queue
+try:
+    from queue import Queue, Empty
+except ImportError:
+    from Queue import Queue, Empty  # NOQA
 
 from circus.util import resolve_name
 from circus.stream.file_stream import FileStream
 from circus.stream.redirector import Redirector
+from zmq.utils.strtypes import u
 
 
 class QueueStream(Queue):
@@ -26,7 +30,7 @@ class StdoutStream(object):
         pass
 
     def __call__(self, data):
-        sys.stdout.write(data['data'])
+        sys.stdout.write(u(data['data']))
         sys.stdout.flush()
 
     def close(self):
@@ -52,7 +56,7 @@ class FancyStdoutStream(StdoutStream):
     You may also configure the timestamp format as defined by
     datetime.strftime. The default is: ::
 
-      %Y-%M-%d %H:%M:%S
+      %Y-%m-%d %H:%M:%S
 
     Here is an example: ::
 
@@ -60,7 +64,7 @@ class FancyStdoutStream(StdoutStream):
       cmd = python -m myapp.server
       stdout_stream.class = FancyStdoutStream
       stdout_stream.color = green
-      stdout_stream.time_format = '%Y/%M/%d | %H:%M:%S'
+      stdout_stream.time_format = '%Y/%m/%d | %H:%M:%S'
     """
 
     # colors in order according to the ascii escape sequences
@@ -74,7 +78,7 @@ class FancyStdoutStream(StdoutStream):
     now = datetime.now
 
     def __init__(self, color=None, time_format=None, *args, **kwargs):
-        self.time_format = time_format or '%Y-%M-%d %H:%M:%S'
+        self.time_format = time_format or '%Y-%m-%d %H:%M:%S'
 
         # If no color is provided we pick one at random
         if color not in self.colors:
@@ -101,7 +105,7 @@ class FancyStdoutStream(StdoutStream):
         return color + prefix
 
     def __call__(self, data):
-        for line in data['data'].split('\n'):
+        for line in u(data['data']).split('\n'):
             if line:
                 self.out.write(self.prefix(data['pid']))
                 self.out.write(line)
@@ -127,10 +131,7 @@ def get_stream(conf):
     else:
         raise ValueError("stream configuration invalid")
 
-    # default refresh_time
-    refresh_time = float(conf.get('refresh_time', 0.3))
-
-    return {'stream': inst, 'refresh_time': refresh_time}
+    return {'stream': inst}
 
 
 def get_pipe_redirector(redirect, extra_info=None, buffer=1024, loop=None):
@@ -154,7 +155,6 @@ def get_pipe_redirector(redirect, extra_info=None, buffer=1024, loop=None):
         return
 
     stream = redirect.get('stream')
-    refresh_time = redirect.get('refresh_time', 0.3)
 
     # finally setup the redirection
-    return Redirector(stream, refresh_time, extra_info, buffer, loop=loop)
+    return Redirector(stream, extra_info, buffer, loop=loop)
