@@ -26,14 +26,17 @@ class RedirectorHandler(object):
                            'name': self.name}
                 datamap.update(self.redirector.extra_info)
                 self.redirector.redirect(datamap)
-        except IOError, ex:
-            if ex[0] != errno.EAGAIN:
+        except IOError as ex:
+            if ex.args[0] != errno.EAGAIN:
                 raise
-            sys.exc_clear()
+            try:
+                sys.exc_clear()
+            except Exception:
+                pass
 
 
 class Redirector(object):
-    def __init__(self, redirect, refresh_time=1.0, extra_info=None,
+    def __init__(self, redirect, extra_info=None,
                  buffer=4096, loop=None):
         self.running = False
         self.pipes = {}
@@ -64,7 +67,7 @@ class Redirector(object):
             del self._active[fd]
 
     def stop(self):
-        for fd in self._active.keys():
+        for fd in list(self._active.keys()):
             self._stop_one(fd)
         self.running = False
 
