@@ -2,10 +2,13 @@ import time
 import tempfile
 import os
 import sys
+import tornado
 
-from circus.tests.support import TestCircus
-from circus.client import CircusClient
+from circus.tests.support import TestCircus, EasyTestSuite
+from circus.client import AsyncCircusClient
 from circus.stream import FileStream
+from circus.py3compat import get_next
+from circus.util import tornado_sleep
 
 
 def run_process(*args, **kw):
@@ -41,44 +44,44 @@ class TestStatsClient(TestCircus):
             if os.path.exists(file):
                 os.remove(file)
 
+    @tornado.testing.gen_test
     def test_handler(self):
-        if os.getenv('TRAVIS', False):
-            return
-
         log = self._get_file()
         stream = {'stream': FileStream(log)}
-        self._run_circus('circus.tests.test_stats_client.run_process',
-                         stdout_stream=stream, stderr_stream=stream,
-                         stats=True)
-
+        cmd = 'circus.tests.test_stats_client.run_process'
+        stdout_stream = stream
+        stderr_stream = stream
+        yield self.start_arbiter(cmd=cmd, stdout_stream=stdout_stream,
+                                 stderr_stream=stderr_stream, stats=True)
         # waiting for data to appear in the file stream
         empty = True
         while empty:
             with open(log) as f:
                 empty = f.read() == ''
-
-            time.sleep(.1)
+            yield tornado_sleep(.1)
 
         # checking that our system is live and running
-        client = CircusClient()
-        res = client.send_message('list')
-        watchers = res['watchers']
-        watchers.sort()
+        client = AsyncCircusClient()
+        res = yield client.send_message('list')
+        watchers = sorted(res['watchers'])
         self.assertEqual(['circusd-stats', 'test'], watchers)
 
         # making sure the stats process run
-        res = client.send_message('status', name='test')
+        res = yield client.send_message('status', name='test')
         self.assertEqual(res['status'], 'active')
 
-        res = client.send_message('status', name='circusd-stats')
+        res = yield client.send_message('status', name='circusd-stats')
         self.assertEqual(res['status'], 'active')
 
         # playing around with the stats now: we should get some !
         from circus.stats.client import StatsClient
         client = StatsClient()
-        next = client.iter_messages().next
+        next = get_next(client.iter_messages())
 
         for i in range(10):
             watcher, pid, stat = next()
             self.assertTrue(watcher in ('test', 'circusd-stats', 'circus'),
                             watcher)
+        yield self.stop_arbiter()
+
+test_suite = EasyTestSuite(__name__)
